
# q-expansion of newform 1134.2.f.d, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1134
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[407, [0, -1]], [325, [1, 0]]]
aps_data = [[-1, 1], [0, 0], [0, -1], [1, -1], [-2, 2], [0, -3], [-1, 0], [2, 0], [0, -2], [-7, 7], [0, -6], [-7, 0], [0, -6], [4, -4], [-6, 6], [-6, 0], [0, -10], [9, -9], [0, -10], [-4, 0], [-11, 0], [6, -6], [10, -10], [15, 0], [2, -2], [2, -2], [0, 16], [0, 0], [-11, 0], [0, -1], [12, 0], [0, -16], [11, -11], [0, 2], [0, 21], [-10, 10], [0, 13], [-16, 0], [0, -22], [-13, 13], [-18, 0], [18, 0], [18, -18], [0, -11], [-1, 0], [-10, 0], [0, 22], [-20, 20], [24, -24], [0, 29], [-19, 0], [0, 6], [7, -7], [-6, 0], [0, 9], [18, -18], [1, 0], [30, 0], [-14, 14], [-29, 29], [0, -16], [0, 15], [16, 0], [0, 14], [-1, 1], [-3, 3], [-20, 20], [0, 18], [0, 12], [-10, 10], [30, -30], [16, 0], [10, -10], [0, -10], [10, 0], [0, -32], [-34, 34], [39, 0], [0, 3], [0, 7], [0, -12], [-9, 9], [12, 0], [5, 0], [12, -12], [22, -22], [34, 0], [41, -41], [-22, 22], [0, -34], [34, 0], [22, -22], [-10, 0], [0, -36], [0, -22], [-6, 0], [0, 18], [42, 0], [-4, 0], [-3, 0], [6, -6], [-17, 0], [0, -8], [27, -27], [0, -18], [-35, 0], [-2, 2], [39, 0], [0, 6], [19, -19], [0, 6], [-18, 0], [0, 7], [20, -20], [38, 0], [-17, 17], [0, -26], [24, 0], [0, -18], [-32, 32], [0, -7], [-23, 23], [6, -6], [-12, 0], [4, -4], [-5, 0], [-5, 5], [-24, 0], [8, -8], [0, -42], [-6, 0], [0, -24], [0, -12], [42, 0], [0, -11], [0, 23], [-35, 0], [0, 24], [0, 3], [1, 0], [34, 0], [-11, 11], [0, -16], [-48, 0], [-2, 0], [-32, 32], [10, -10], [-3, 3], [0, 52], [-14, 0], [0, 23], [-18, 0], [-10, 0], [0, -42], [12, -12], [-12, 12], [10, 0], [33, -33], [-43, 0], [0, -25], [-4, 4], [-19, 0], [0, 22], [-42, 0], [0, -18], [60, -60], [62, 0], [-23, 23], [-13, 0], [0, 42], [-4, 4], [0, 5], [0, -30], [-27, 27], [0, 24], [0, 50], [12, -12], [-61, 0], [-52, 0], [-23, 23], [-36, 36], [-8, 8], [0, -25], [22, 0], [0, 44], [-45, 45], [-3, 0], [10, -10], [0, 42], [18, 0], [-59, 0], [36, -36], [22, 0], [-5, 5], [48, 0], [0, -38], [0, 61], [26, -26], [33, -33], [-54, 0], [0, -63], [66, -66], [0, -50], [27, -27], [-6, 0], [54, 0], [-46, 0], [0, 54], [54, -54], [0, 28], [2, 0], [0, -1], [0, 0], [20, -20], [0, -36], [-50, 50], [0, 68], [37, -37], [8, 0], [0, 25], [0, 23], [0, -10], [0, -9], [40, 0], [0, -48], [-5, 5], [10, -10], [28, 0], [36, -36], [-54, 54], [0, -33], [4, 0], [0, -22], [0, -45], [-26, 26], [-48, 48], [0, -73], [-54, 0], [0, -6], [-50, 0], [-14, 14], [-70, 0], [0, -8], [18, 0], [0, -62], [6, -6], [-12, 0], [0, 40], [0, -44], [66, 0], [0, -49], [39, 0], [0, -12], [-37, 37], [-29, 29], [26, 0], [77, 0], [-18, 18], [37, 0], [-22, 0], [14, -14], [-22, 22], [0, -67], [-2, 0], [0, -78], [26, -26], [-45, 0], [41, -41], [0, -16], [0, -3], [0, -63], [2, 0], [54, -54], [0, -8], [0, -25], [-72, 0], [0, 82], [-33, 33], [43, 0], [-72, 72], [0, 54], [0, 30], [-12, 12], [-35, 35], [0, -42], [68, 0], [-21, 0], [0, -50], [34, -34], [51, 0], [-42, 42], [42, 0], [0, -45], [0, -70], [-41, 41], [0, -33], [46, -46], [39, -39], [-60, 0], [-36, 36], [0, -13], [-18, 0], [16, 0]]
